package hpdos_rdma_offloaded;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.util.Properties;

import com.ibm.darpc.DaRPCServerEndpoint;
import com.ibm.darpc.DaRPCServerGroup;
import com.ibm.disni.RdmaServerEndpoint;

import org.rocksdb.RocksDB;
import org.rocksdb.RocksDBException;

import hpdos_rdma_offloaded.handler.NetworkHandler;
import hpdos_rdma_offloaded.handler.RegistrationHandler;
import hpdos_rdma_offloaded.handler.StorageHandler;
import hpdos_rdma_offloaded.protocol.Request;
import hpdos_rdma_offloaded.protocol.Response;
import hpdos_rdma_offloaded.service.FollowerMetadataService;
import hpdos_rdma_offloaded.service.MasterMetadataServerService;

public class MetadataServer implements Runnable{
    private String host;
    private int poolsize;
    private int recvQueue;
    private int sendQueue;
    private int clusterSize;
    private int wqSize = recvQueue;
    private boolean polling;
    private int maxinline = 0;
    private int rdma_port;
    private long[] clusterAffinities;

    boolean isMaster;
    StorageHandler storageHandler;
    NetworkHandler networkHandler;
    MasterMetadataServerService metadataServerService = null;
    FollowerMetadataService followerMetadataServerService = null;
    RegistrationHandler registrationHandler = null;
    public final static Properties properties= new Properties();
    String propertyPath;

    public MetadataServer(String propertyPath) throws Exception,RocksDBException {
        this.propertyPath = propertyPath;
        InputStream inputStream = new FileInputStream(propertyPath);
        properties.load(inputStream);
        this.isMaster = Boolean.valueOf((String)properties.get("app.MASTER")); 
        

        host =  properties.getProperty("app.HOST");
        poolsize = Integer.valueOf( properties.getProperty("app.cpu_affinity"));
        recvQueue = Integer.valueOf(properties.getProperty("app.rdma_receive_queue"));
        sendQueue = Integer.valueOf( properties.getProperty("app.rdma_send_queue"));
        clusterSize = Integer.valueOf(properties.getProperty("app.rdma_cluster_size"));
        wqSize = recvQueue;
        maxinline = Integer.valueOf(properties.getProperty("app.rdma_max_inline"));
        rdma_port = Integer.valueOf(properties.getProperty("app.rdma_server_port"));
        polling = Boolean.valueOf(properties.getProperty("app.rdma_polling"));
        int registrationPort = Integer.valueOf(properties.getProperty("app.follower_registration_port"));
        
        clusterAffinities = new long[poolsize];
        for (int i = 0; i < poolsize; i++)
        {
            long cpu = 1L << i;
            clusterAffinities[i] = cpu;
            System.out.println(cpu);
        }

        isMaster = Boolean.valueOf(properties.getProperty("app.MASTER"));
        storageHandler = new StorageHandler(properties.getProperty("app.db_path"));
        networkHandler = new NetworkHandler(isMaster);
        // metadataServerService =  new MasterMetadataServerService(this.networkHandler,storageHandler);
       registrationHandler = new RegistrationHandler(isMaster, host, registrationPort, networkHandler);
       if(isMaster)
        {
            
            System.out.println("STARTING MASTER SERVICES.");
            metadataServerService =  new MasterMetadataServerService(this.networkHandler,storageHandler);
        }
        else
        {
            System.out.println("STARTING FOLLOWER SERVICES.");
            followerMetadataServerService =  new FollowerMetadataService(storageHandler);
        }

    }

    public void run(){
        // MetadataServerService rpcService = new MetadataServerService();
        DaRPCServerGroup<Request, Response> group = null;
        try
        {
            if (isMaster) {
                group = DaRPCServerGroup.createServerGroup(this.metadataServerService, this.clusterAffinities, 1000, maxinline, polling, recvQueue, sendQueue, wqSize, clusterSize);        
            } else {
                group = DaRPCServerGroup.createServerGroup(this.followerMetadataServerService, this.clusterAffinities, 1000, maxinline, polling, recvQueue, sendQueue, wqSize, clusterSize);    
            }
            
            RdmaServerEndpoint<DaRPCServerEndpoint<Request, Response>> serverEp = null;
            serverEp = group.createServerEndpoint();
            InetSocketAddress address = new InetSocketAddress(host, rdma_port);
            serverEp.bind(address, 100);
            while(true){
                try
                {
                // System.out.println("Listening to RDMA requests a, IP: " + host + " , PORT: " + rdma_port);
                    serverEp.accept();
                // System.out.println("Accepted connection.");
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }
            }
        }catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    public static void main(String[] args)  throws Exception
    {
            RocksDB.loadLibrary();
            //var process = Runtime.getRuntime().exec("/bin/sh export LD_LIBRARY_PATH=/usr/local/lib");
            //process.wait();
           // System.out.println("Exit value "+process.exitValue());
            Runnable r = new MetadataServer(args[0]);
            new Thread(r).start();
    }
}
