package hpdos_rdma_offloaded.handler;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.lang.ClassNotFoundException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import hpdos_rdma_offloaded.lib.Follower;

public class RegistrationHandler {
    public ServerSocket server;
    public int port = 9876;
    boolean isMaster;
    String registrationServerIp;
    int follower_registration_port;
    int sal_registration_port;
    NetworkHandler networkHandler;

    public RegistrationHandler(boolean isMaster, String registrationServerIP, int registrationPort, NetworkHandler networkHandler) throws IOException {
        this.registrationServerIp = registrationServerIP;
        this.follower_registration_port = 9876;
        this.sal_registration_port = 9875;
        this.networkHandler = networkHandler;
        if (isMaster)
        {
            
            startFollowerRegistrationHandlerServer();
            startSalRegistrationHandlerServer();    
        }
        else {
            // Follower follower = new Follower();
            // follower.setFollowerId( UUID.randomUUID().toString());
            // follower.setIpAddress("192.168.200.20");
            String followerUUID = UUID.randomUUID().toString();
            String followerIp = registrationServerIP;
            String message = followerUUID + ";" + followerIp;
            sendRegistrationRequest(message);
        }
    }

    private void sendRegistrationRequest(String message) throws IOException {
        System.out.println("Sending a registration request with message: "+message);
        InetAddress host = InetAddress.getByName(this.registrationServerIp);
        Socket socket = null;
        ObjectOutputStream oos = null;
        socket = new Socket(this.registrationServerIp, 9876);
        oos = new ObjectOutputStream(socket.getOutputStream());
        oos.writeObject(message);
    }

    public void startFollowerRegistrationHandlerServer(){
        Runnable serverTask = new Runnable() {
            @Override
            public void run()
            {
                try
                {
                    InetAddress addr = InetAddress.getByName(registrationServerIp);
                    ServerSocket serverSocket = new ServerSocket(follower_registration_port, 50, addr);
                    System.out.println("Started the follower registration service...");
                    while (true) {
                        Socket socket = serverSocket.accept();
                        System.out.println("Accepted connection");
                        ObjectInputStream ois = new ObjectInputStream(socket.getInputStream());
                        String message = (String) ois.readObject();
                        String arr[] = message.split(";");
                        String followerUUID = arr[0];
                        String followerIp = arr[1];
                        System.out.println("Got follower ip: " + followerIp + " , with UUID of: " + followerUUID);
                        
                       NetworkHandler networkHandler = new NetworkHandler();
                       networkHandler.connectToFollower(followerUUID, followerIp);
                        System.out.println("RDMA connection establised");
                        // Add the follower details to the respective class/component
                        // clientProcessingPool.submit(new ClientTask(clientSocket));
                    }
                } catch (Exception e) {
                    System.err.println("Unable to process client request");
                    e.printStackTrace();
                }
            }
        };
        Thread serverThread = new Thread(serverTask);
        serverThread.start();
    }

    public void startSalRegistrationHandlerServer(){
        Runnable serverTask = new Runnable() {
            @Override
            public void run() {
                try {
                    InetAddress addr = InetAddress.getByName(registrationServerIp);
                    ServerSocket serverSocket = new ServerSocket(sal_registration_port, 50, addr);
                    System.out.println("Started the SAL registration service...");
                    while (true) {
                        Socket socket = serverSocket.accept();
                        ObjectInputStream ois = new ObjectInputStream(socket.getInputStream());
                        OutputStream outputStream = socket.getOutputStream();
                        ObjectOutputStream objectOutputStream = new ObjectOutputStream(outputStream);
                        String ip = (String) ois.readObject();
                        String ips = registrationServerIp + ";";
                        for (Follower follower : NetworkHandler.followers) {
                            ips = ips + follower.getIpAdress() + ";";
                        }
                        objectOutputStream.writeObject(ips);
                        NetworkHandler networkHandler = new NetworkHandler();
                        networkHandler.connectToSal(ip);
                        // clientProcessingPool.submit(new ClientTask(clientSocket));
                    }
                } catch (Exception e) {
                    System.err.println("Unable to process client request");
                    e.printStackTrace();
                }
            }
        };
        Thread serverThread = new Thread(serverTask);
        serverThread.start();
    }
}
