package hpdos_rdma_offloaded.service;

import java.io.IOException;

import com.ibm.darpc.DaRPCClientGroup;
import com.ibm.darpc.DaRPCServerEndpoint;
import com.ibm.darpc.DaRPCServerEvent;
import com.ibm.darpc.DaRPCService;

import org.rocksdb.RocksDBException;

import hpdos_rdma_offloaded.handler.StorageHandler;
import hpdos_rdma_offloaded.lib.Packet;
import hpdos_rdma_offloaded.protocol.AckType;
import hpdos_rdma_offloaded.protocol.InvalidationRequest;
import hpdos_rdma_offloaded.protocol.InvalidationResponse;
import hpdos_rdma_offloaded.protocol.Request;
import hpdos_rdma_offloaded.protocol.RequestType;
import hpdos_rdma_offloaded.protocol.Response;
import hpdos_rdma_offloaded.protocol.RpcProtocol;

public class FollowerMetadataService extends RpcProtocol implements DaRPCService<Request, Response>{
    StorageHandler storageHandler;
    DaRPCClientGroup<InvalidationRequest, InvalidationResponse> invalidationClientGroup;
    public FollowerMetadataService(StorageHandler storageHandler) throws Exception{
        System.out.println("New MetadataServerService Created");
        this.storageHandler  = storageHandler;
    }

    @Override
    public void processServerEvent(DaRPCServerEvent<Request, Response> event) throws IOException {
        Request request = event.getReceiveMessage();
        Response response = event.getSendMessage();
        
        System.out.print("Received "+request.getRequestType()+" Request for key: " + new String(request.getKey()));
        if(request.getValue() != null)
            System.out.print(" value : "+ new String(request.getValue()) );
        System.out.println();
        
        try{
            if (request.getRequestType() == RequestType.PUT) {
                this.storageHandler.create(request);
                response.setAck(AckType.SUCCESS);
                response.setKey(null);
                response.setValue(null);
            } 
            else if(request.getRequestType() == RequestType.GET) {
                byte[] value = this.storageHandler.read(request); 
                if(value == null)
                {
                    response.setAck(AckType.NOTFOUND);
                }
                else
                {
                    response.setAck(AckType.SUCCESS_WITH_VALUE);
                    response.setValue(value);
                }
            }
            else if(request.getRequestType() == RequestType.DELETE){
                this.storageHandler.delete(request);
                response.setAck(AckType.SUCCESS);
            }
            else{
                response.setAck(AckType.INVALID_REQUEST);
            }
        }
        catch(RocksDBException ex){
                ex.printStackTrace();
                response.setAck(AckType.DBFAILED);
        }
        event.triggerResponse();
    }

    @Override
    public void open(DaRPCServerEndpoint<Request, Response> rpcClientEndpoint) {
        System.out.print("Received new connection  ");
    }

    @Override
    public void close(DaRPCServerEndpoint<Request, Response> rpcClientEndpoint) {
        System.out.println("Closing Connection");   
    }
    
}
